@@ -1,7 +1,14 @@
-
 #' @export
-vec_proxy_compare.integer64 <- function(x, ...) {
-  bit64::rank.integer64(x)
+vec_proxy_equal.integer64 <- function(x, ...) {
+  if (is.array(x)) {
+    # Stopgap to convert arrays to data frames, then run them through
+    # `vec_proxy_equal()` again, which will proxy each column
+    x <- as_data_frame_from_array(x)
+    x <- vec_proxy_equal(x)
+    return(x)
+  }
+
+  integer64_proxy(x)
 }
 
 # Print -------------------------------------------------------------------
@@ -108,3 +115,46 @@ vec_cast.integer64.double <- function(x, to, ...) {
 vec_cast.double.integer64 <- function(x, to, ...) {
   as.double(x)
 }
+
+# ------------------------------------------------------------------------------
+
+integer64_proxy <- function(x) {
+  .Call(vctrs_integer64_proxy, x)
+}
+integer64_restore <- function(x) {
+  .Call(vctrs_integer64_restore, x)
+}
+
+# ------------------------------------------------------------------------------
+
+as_data_frame_from_array <- function(x) {
+  # Alternative to `as.data.frame.array()` that always strips 1-D arrays
+  # of their dimensions. Unlike `as.data.frame2()`, it doesn't unclass the
+  # input, which means that each column retains its original class.
+  # This function doesn't attempt to keep the names of `x` at all.
+
+  dim <- dim(x)
+  n_dim <- length(dim)
+
+  if (n_dim == 1) {
+    # Treat 1-D arrays as 1 column matrices
+    dim(x) <- c(dim, 1L)
+    n_dim <- 2L
+  }
+
+  n_row <- dim[[1L]]
+  n_col <- prod(dim[-1L])
+  n_col_seq <- seq_len(n_col)
+
+  dim(x) <- c(n_row, n_col)
+
+  out <- vector("list", n_col)
+  names(out) <- as_unique_names(rep("", n_col), quiet = TRUE)
+
+  for (i in n_col_seq) {
+    out[[i]] <- x[, i, drop = TRUE]
+  }
+
+  new_data_frame(out, n = n_row)
+}
+
