@@ -383,24 +383,24 @@ rational(2, 4) < rational(2, 3)
 sort(x)
 
 ## -----------------------------------------------------------------------------
-new_poly <- function(x) {
-  new_list_of(x, ptype = integer(), class = "vctrs_poly")
-}
-
 poly <- function(...) {
-  x <- list(...)
-  x <- lapply(x, vec_cast, integer())
+  x <- vec_cast_common(..., .to = integer())
   new_poly(x)
 }
+new_poly <- function(x) {
+  new_list_of(x, ptype = integer(), class = "vctrs_poly_list")
+}
 
-vec_ptype_full.vctrs_poly <- function(x, ...) "polynomial"
-vec_ptype_abbr.vctrs_poly <- function(x, ...) "poly"
+vec_ptype_full.vctrs_poly_list <- function(x, ...) "polynomial"
+vec_ptype_abbr.vctrs_poly_list <- function(x, ...) "poly"
 
-format.vctrs_poly <- function(x, ...) {
+format.vctrs_poly_list <- function(x, ...) {
   format_one <- function(x) {
     if (length(x) == 0) {
       return("")
-    } else if (length(x) == 1) {
+    }
+
+    if (length(x) == 1) {
       format(x)
     } else {
       suffix <- c(paste0("\u22C5x^", seq(length(x) - 1, 1)), "")
@@ -409,13 +409,14 @@ format.vctrs_poly <- function(x, ...) {
       paste0(out, collapse = " + ")
     }
   }
+
   vapply(x, format_one, character(1))
 }
 
-obj_print_data.vctrs_poly <- function(x, ...) {
-  if (length(x) == 0)
-    return()
-  print(format(x), quote = FALSE)
+obj_print_data.vctrs_poly_list <- function(x, ...) {
+  if (length(x) != 0) {
+    print(format(x), quote = FALSE)
+  }
 }
 
 p <- poly(1, c(1, 0, 0, 0, 2), c(1, 0, 1))
@@ -426,6 +427,29 @@ class(p)
 p[2]
 p[[2]]
 
+## -----------------------------------------------------------------------------
+vec_is_list(p)
+
+## -----------------------------------------------------------------------------
+poly <- function(...) {
+  x <- vec_cast_common(..., .to = integer())
+  x <- new_poly(x)
+  new_rcrd(list(data = x), class = "vctrs_poly")
+}
+format.vctrs_poly <- function(x, ...) {
+  format(field(x, "data"))
+}
+
+## -----------------------------------------------------------------------------
+p <- poly(1, c(1, 0, 0, 0, 2), c(1, 0, 1))
+p
+
+## -----------------------------------------------------------------------------
+vec_is_list(p)
+
+## -----------------------------------------------------------------------------
+p[[2]]
+
 ## -----------------------------------------------------------------------------
 p == poly(c(1, 0, 1))
 
@@ -434,7 +458,9 @@ p < p[2]
 
 ## -----------------------------------------------------------------------------
 vec_proxy_compare.vctrs_poly <- function(x, ...) {
-  x_raw <- vec_data(x)
+  # Get the list inside the record vector
+  x_raw <- vec_data(field(x, "data"))
+
   # First figure out the maximum length
   n <- max(vapply(x_raw, length, integer(1)))
 
@@ -634,6 +660,32 @@ is_percent <- function(x) {
 #  #' @export
 #  vec_cast.double.pizza_percent <- function(x, to, ...) vec_data(x)
 
+## ---- eval=FALSE--------------------------------------------------------------
+#  #' @export
+#  #' @method vec_arith my_type
+#  vec_arith.my_type <- function(op, x, y, ...) {
+#    UseMethod("vec_arith.my_type", y)
+#  }
+
+## ---- eval=FALSE--------------------------------------------------------------
+#  #' @export
+#  #' @method vec_arith.my_type my_type
+#  vec_arith.my_type.my_type <- function(op, x, y, ...) {
+#    # implementation here
+#  }
+#  
+#  #' @export
+#  #' @method vec_arith.my_type integer
+#  vec_arith.my_type.integer <- function(op, x, y, ...) {
+#    # implementation here
+#  }
+#  
+#  #' @export
+#  #' @method vec_arith.integer my_type
+#  vec_arith.integer.my_type <- function(op, x, y, ...) {
+#    # implementation here
+#  }
+
 ## ---- eval = FALSE------------------------------------------------------------
 #  expect_error(vec_c(1, "a"), class = "vctrs_error_incompatible_type")
 
