@@ -13,6 +13,19 @@ struct cast_opts {
   struct fallback_opts fallback;
 };
 
+// FIXME: Should we merge these two structs?
+static inline
+struct ptype2_opts cast_opts_as_ptype2_opts(const struct cast_opts* p_opts) {
+  return (struct ptype2_opts) {
+    .x = p_opts->x,
+    .y = p_opts->to,
+    .p_x_arg = p_opts->p_x_arg,
+    .p_y_arg = p_opts->p_to_arg,
+    .call = p_opts->call,
+    .fallback = p_opts->fallback,
+  };
+}
+
 struct cast_common_opts {
   struct vctrs_arg* p_arg;
   struct r_lazy call;
@@ -42,15 +55,15 @@ r_obj* vec_cast_params(r_obj* x,
                        r_obj* to,
                        struct vctrs_arg* p_x_arg,
                        struct vctrs_arg* p_to_arg,
-                       enum df_fallback df_fallback,
+                       struct r_lazy call,
                        enum s3_fallback s3_fallback) {
   const struct cast_opts opts = {
     .x = x,
     .to = to,
     .p_x_arg = p_x_arg,
     .p_to_arg = p_to_arg,
+    .call = call,
     .fallback = {
-      .df = df_fallback,
       .s3 = s3_fallback
     }
   };
@@ -68,7 +81,6 @@ r_obj* vec_cast_common_opts(r_obj* xs,
 
 r_obj* vec_cast_common_params(r_obj* xs,
                               r_obj* to,
-                              enum df_fallback df_fallback,
                               enum s3_fallback s3_fallback,
                               struct vctrs_arg* p_arg,
                               struct r_lazy call);
@@ -84,11 +96,11 @@ r_obj* vec_cast_e(const struct cast_opts* opts,
                   ERR* err);
 
 r_obj* vec_cast_default(r_obj* x,
-                        r_obj* y,
-                        r_obj* p_x_arg,
-                        r_obj* p_to_arg,
+                        r_obj* to,
+                        struct vctrs_arg* p_x_arg,
+                        struct vctrs_arg* p_to_arg,
                         struct r_lazy call,
-                        const struct fallback_opts* opts);
+                        const struct fallback_opts* p_opts);
 
 
 #endif
